/*
    semu - Sega Saturn Emulator
    Copyright (C) 1998  Chuck Mason

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Chuck Mason IV                      chuckjr@sundial.net
    477 Moffat Loop
    Oviedo, Fl 32765
*/

#ifndef __MEMORY_H
#define __MEMORY_H

#include <config.h>
#include <semu_types.h>
#include "sh2proc.h"

typedef enum {MEM_BYTE, MEM_WORD, MEM_LONG} mem_access;

uint32_t sh2_read_memory(sh2_processor *, uint32_t, mem_access);
void sh2_write_memory(sh2_processor *, uint32_t, uint32_t, mem_access);
void sh2_write_byte_rom(uint32_t, uint8_t);

static inline uint8_t sh2_read_byte(sh2_processor *sh2, uint32_t addr)
{
	return sh2_read_memory(sh2, addr, MEM_BYTE);
}

static inline uint16_t sh2_read_word(sh2_processor *sh2, uint32_t addr)
{
	return sh2_read_memory(sh2, addr, MEM_WORD);
}

static inline uint32_t sh2_read_long(sh2_processor *sh2, uint32_t addr)
{
	return sh2_read_memory(sh2, addr, MEM_LONG);
}

static inline void sh2_write_byte(sh2_processor *sh2, uint32_t addr, uint8_t val)
{
	sh2_write_memory(sh2, addr, val, MEM_BYTE);
}

static inline void sh2_write_word(sh2_processor *sh2, uint32_t addr, uint16_t val)
{
	sh2_write_memory(sh2, addr, val, MEM_WORD);
}

static inline void sh2_write_long(sh2_processor *sh2, uint32_t addr, uint32_t val)
{
	sh2_write_memory(sh2, addr, val, MEM_LONG);
}

static inline uint32_t direct_read(uint8_t *addr, mem_access type)
{
	switch(type) {
	case MEM_BYTE:
		return *addr;
	case MEM_WORD:
		return (addr[0] << 8)|addr[1];
	case MEM_LONG:
		return (addr[0] << 24)|(addr[1] << 16)|(addr[2] << 8)|addr[3];
	}
}

static inline void direct_write(uint8_t *addr, uint32_t val, mem_access type)
{
	switch(type) {
	case MEM_BYTE:
		*addr = val;
		break;
	case MEM_WORD:
		addr[0] = val >> 8;
		addr[1] = val;
		break;
	case MEM_LONG:
		addr[0] = val >> 24;
		addr[1] = val >> 16;
		addr[2] = val >> 8;
		addr[3] = val;
		break;
	}
}

#endif
