/*
    semu - Sega Saturn Emulator
    Copyright (C) 1998  Chuck Mason

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Chuck Mason IV                      chuckjr@sundial.net
    477 Moffat Loop
    Oviedo, Fl 32765
*/

#ifndef __OPCODES_H
#define __OPCODES_H

#include <semu_types.h>

#include "sh2proc.h"

void sh2_dump_instruction(sh2_processor *, uint32_t);

/* ************************** */

void sh2_opcode_nop(sh2_processor *);
void sh2_opcode_add(sh2_processor *);
void sh2_opcode_addi(sh2_processor *);
void sh2_opcode_addc(sh2_processor *);
void sh2_opcode_addv(sh2_processor *);
void sh2_opcode_and(sh2_processor *);
void sh2_opcode_andi(sh2_processor *);
void sh2_opcode_andm(sh2_processor *);
void sh2_opcode_bf(sh2_processor *);
void sh2_opcode_bt(sh2_processor *);
void sh2_opcode_clrmac(sh2_processor *);

void sh2_opcode_clrt(sh2_processor *);
void sh2_opcode_sett(sh2_processor *);

void sh2_opcode_cmpeq(sh2_processor *);
void sh2_opcode_cmpge(sh2_processor *);
void sh2_opcode_cmpgt(sh2_processor *);
void sh2_opcode_cmphi(sh2_processor *);
void sh2_opcode_cmphs(sh2_processor *);
void sh2_opcode_cmppl(sh2_processor *);
void sh2_opcode_cmppz(sh2_processor *);
void sh2_opcode_cmpstr(sh2_processor *);
void sh2_opcode_cmpim(sh2_processor *);

void sh2_opcode_div0s(sh2_processor *);
void sh2_opcode_div0u(sh2_processor *);
void sh2_opcode_div1(sh2_processor *);

void sh2_opcode_dmuls(sh2_processor *);
void sh2_opcode_dmulu(sh2_processor *);

void sh2_opcode_dt(sh2_processor *);

void sh2_opcode_extsb(sh2_processor *);
void sh2_opcode_extsw(sh2_processor *);
void sh2_opcode_extub(sh2_processor *);
void sh2_opcode_extuw(sh2_processor *);

void sh2_opcode_sleep(sh2_processor *);
void sh2_opcode_invalid(sh2_processor *);

void sh2_opcode_bfs(sh2_processor *);
void sh2_opcode_bts(sh2_processor *);

void sh2_opcode_bra(sh2_processor *);
void sh2_opcode_braf(sh2_processor *);
void sh2_opcode_bsr(sh2_processor *);
void sh2_opcode_bsrf(sh2_processor *);

void sh2_opcode_jmp(sh2_processor *);
void sh2_opcode_jsr(sh2_processor *);

void sh2_opcode_ldcsr(sh2_processor *);
void sh2_opcode_ldcgbr(sh2_processor *);
void sh2_opcode_ldcvbr(sh2_processor *);
void sh2_opcode_ldcmsr(sh2_processor *);
void sh2_opcode_ldcmgbr(sh2_processor *);
void sh2_opcode_ldcmvbr(sh2_processor *);

void sh2_opcode_ldsmach(sh2_processor *);
void sh2_opcode_ldsmacl(sh2_processor *);
void sh2_opcode_ldspr(sh2_processor *);
void sh2_opcode_ldsmmach(sh2_processor *);
void sh2_opcode_ldsmmacl(sh2_processor *);
void sh2_opcode_ldsmpr(sh2_processor *);

void sh2_opcode_macl(sh2_processor *);
void sh2_opcode_macw(sh2_processor *);

/* MOV Series!!#@$!@#$ */
void sh2_opcode_mov(sh2_processor *);
void sh2_opcode_movbs(sh2_processor *);
void sh2_opcode_movws(sh2_processor *);
void sh2_opcode_movls(sh2_processor *);
void sh2_opcode_movbl(sh2_processor *);
void sh2_opcode_movwl(sh2_processor *);
void sh2_opcode_movll(sh2_processor *);
void sh2_opcode_movbm(sh2_processor *);
void sh2_opcode_movwm(sh2_processor *);
void sh2_opcode_movlm(sh2_processor *);
void sh2_opcode_movi(sh2_processor *);
void sh2_opcode_movwi(sh2_processor *);
void sh2_opcode_movli(sh2_processor *);
void sh2_opcode_movbp(sh2_processor *);
void sh2_opcode_movwp(sh2_processor *);
void sh2_opcode_movlp(sh2_processor *);
void sh2_opcode_movbs0(sh2_processor *);
void sh2_opcode_movws0(sh2_processor *);
void sh2_opcode_movls0(sh2_processor *);
void sh2_opcode_movbl0(sh2_processor *);
void sh2_opcode_movwl0(sh2_processor *);
void sh2_opcode_movll0(sh2_processor *);

void sh2_opcode_movblg(sh2_processor *);
void sh2_opcode_movwlg(sh2_processor *);
void sh2_opcode_movllg(sh2_processor *);
void sh2_opcode_movbsg(sh2_processor *);
void sh2_opcode_movwsg(sh2_processor *);
void sh2_opcode_movlsg(sh2_processor *);

void sh2_opcode_movbs4(sh2_processor *);
void sh2_opcode_movws4(sh2_processor *);
void sh2_opcode_movls4(sh2_processor *);
void sh2_opcode_movbl4(sh2_processor *);
void sh2_opcode_movwl4(sh2_processor *);
void sh2_opcode_movll4(sh2_processor *);

void sh2_opcode_mova(sh2_processor *);

void sh2_opcode_movt(sh2_processor *);
/* ------------------------ That's it for the MOV series!! */
void sh2_opcode_mull(sh2_processor *);
void sh2_opcode_muls(sh2_processor *);
void sh2_opcode_mulu(sh2_processor *);

void sh2_opcode_neg(sh2_processor *);
void sh2_opcode_negc(sh2_processor *);

void sh2_opcode_not(sh2_processor *);

void sh2_opcode_or(sh2_processor *);
void sh2_opcode_ori(sh2_processor *);
void sh2_opcode_orm(sh2_processor *);

void sh2_opcode_rotcl(sh2_processor *);
void sh2_opcode_rotcr(sh2_processor *);

void sh2_opcode_rotl(sh2_processor *);
void sh2_opcode_rotr(sh2_processor *);

void sh2_opcode_rte(sh2_processor *);
void sh2_opcode_rts(sh2_processor *);

void sh2_opcode_shal(sh2_processor *);
void sh2_opcode_shar(sh2_processor *);

void sh2_opcode_shll2(sh2_processor *);
void sh2_opcode_shll8(sh2_processor *);
void sh2_opcode_shll16(sh2_processor *);

void sh2_opcode_shlr(sh2_processor *);

void sh2_opcode_shlr2(sh2_processor *);
void sh2_opcode_shlr8(sh2_processor *);
void sh2_opcode_shlr16(sh2_processor *);

void sh2_opcode_stcsr(sh2_processor *);
void sh2_opcode_stcgbr(sh2_processor *);
void sh2_opcode_stcvbr(sh2_processor *);
void sh2_opcode_stcmsr(sh2_processor *);
void sh2_opcode_stcmgbr(sh2_processor *);
void sh2_opcode_stcmvbr(sh2_processor *);

void sh2_opcode_stsmach(sh2_processor *);
void sh2_opcode_stsmacl(sh2_processor *);
void sh2_opcode_stspr(sh2_processor *);
void sh2_opcode_stsmmach(sh2_processor *);
void sh2_opcode_stsmmacl(sh2_processor *);
void sh2_opcode_stsmpr(sh2_processor *);

void sh2_opcode_sub(sh2_processor *);
void sh2_opcode_subc(sh2_processor *);
void sh2_opcode_subv(sh2_processor *);

void sh2_opcode_swapb(sh2_processor *);
void sh2_opcode_swapw(sh2_processor *);

void sh2_opcode_tas(sh2_processor *);

void sh2_opcode_trapa(sh2_processor *);

void sh2_opcode_tst(sh2_processor *);
void sh2_opcode_tsti(sh2_processor *);
void sh2_opcode_tstm(sh2_processor *);

void sh2_opcode_xor(sh2_processor *);
void sh2_opcode_xori(sh2_processor *);
void sh2_opcode_xorm(sh2_processor *);

void sh2_opcode_xtrct(sh2_processor *);

#endif
