/*
    semu - Sega Saturn Emulator
    Copyright (C) 1998  Chuck Mason

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Chuck Mason IV                      chuckjr@sundial.net
    477 Moffat Loop
    Oviedo, Fl 32765
*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>

#include "main.h"
#include "header.h"
#include "game.h"
#include "bios.h"
#include "error.h"

/* #define DONT_CHECK_CONTROLS /* */

int read_header(int fd)
{
	saturn_header game_header;

	/* This will clear all that bullox that i sometimes get .. ha */
	memset(&game_header, 0, sizeof(game_header));	

	if((read(fd, (char *)&game_header, sizeof(game_header))) != sizeof(game_header))
		return error(ERR_NOREAD_HEADER);
		
	current_game.header = game_header;
	
	if((read(fd, (char *)&current_game.security_code, SECURITY_CODE_LEN)) != SECURITY_CODE_LEN)
		return error(ERR_NOREAD_HEADER);

	if((read(fd, (char *)&current_game.area_codes, AREA_CODE_LEN * 8)) != (AREA_CODE_LEN * 8))
		return error(ERR_NOREAD_HEADER);
				
	return RET_SUCCESS;
}

int read_aip(int fd)
{
	printf("Loading AIP, this make take a while, please wait...");
	fflush(stdout);

	if((read(fd, (char *)current_game.application_initial_program, APPLICATION_INITIAL_PROGRAM_SIZE)) != APPLICATION_INITIAL_PROGRAM_SIZE)
		return error(ERR_NOREAD_AIP);
		
	printf("done.\n");
	
	return RET_SUCCESS;
}

int validify_header()
{
	if(strncmp(current_game.header.hardware, SEGA_MAGIC, strlen(SEGA_MAGIC)) != 0) 
		return error(ERR_INVALID_HEADER);

	if(current_game.header.version[0] != 'V')
		error("Silly error: version does not begin with a \'V\'!\n");

	sscanf(current_game.header.device_info, "CD-%d/%d", 
					&current_game.current_cd, 
					&current_game.max_cds);

	if(current_game.current_cd != 1)  /* My protection for now.. this is prolly gonna */
		error("Not 1st cd!\n");               /*   be removed later */

#if 0
	if(strnchr(current_game.header.area, 'U', AREA_LEN) == NULL)
	  return error(ERR_ONLY_US);
#endif

#ifndef DONT_CHECK_CONTROLS
	{
		char parifs[] = "AMKST";
		char *parifnames[5] = {
			"Analog Joystick", "Mouse", "Keyboard", "Steering Wheel",
			"Multitap"
		};
		int i, parifcount = 5;
		
		for(i = 0; i < parifcount; i++) 
			if(strnchr(current_game.header.parif, parifs[i], PARIF_LEN) != NULL)
				error("Device: %s used but not emulated.\n", parifnames[i]);
	}
#endif
	return RET_SUCCESS;
}

void display_header()
{
	printf("Hardware: ");
	fwrite(current_game.header.hardware, HARDWARE_LEN, 1, stdout);

	printf("\nMaker ID: ");
	fwrite(current_game.header.maker_id, MAKER_ID_LEN, 1, stdout);

	printf("\nProduct Number: ");
	fwrite(current_game.header.product_number, PRODUCT_NUMBER_LEN, 1, stdout);

	printf("\nVersion: ");
	fwrite(current_game.header.version, VERSION_LEN, 1, stdout);

	printf("\nRelease: ");
	fwrite(current_game.header.release, RELEASE_LEN, 1, stdout);

	printf(" (%s)\nDevice Info: ", strdate(current_game.header.release));
	fwrite(current_game.header.device_info, DEVICE_INFO_LEN, 1, stdout);

	printf("\nArea Codes: ");
	fwrite(current_game.header.area, AREA_LEN, 1, stdout);

	printf("\nPariphreals(sp?): ");
	fwrite(current_game.header.parif, PARIF_LEN, 1, stdout);

	printf("\nTitle: %s\n", 
		fmttitle(current_game.header.title));

	printf("IP Size: $%08X (%lu)\n", 
			current_game.header.ip_size, 
			current_game.header.ip_size);
	printf("Master SP: $%08X (%lu)\n", 
			current_game.header.stack_m,
			current_game.header.stack_m);
	printf("Slave  SP: $%08X (%lu)\n", 
			current_game.header.stack_s,
			current_game.header.stack_s);
	printf("First Read Address: $%08X (%lu)\n", 
			current_game.header.first_read,
			current_game.header.first_read);
	printf("First Read Size: $%08X (%lu)\n", 
			current_game.header.first_read_size,
			current_game.header.first_read_size);
	return;
}
