/*
    semu - Sega Saturn Emulator
    Copyright (C) 1998  Chuck Mason

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Chuck Mason IV                      chuckjr@sundial.net
    477 Moffat Loop
    Oviedo, Fl 32765
*/

#ifndef __HEADER_H
#define __HEADER_H

#define HARDWARE_LEN 0x10
#define MAKER_ID_LEN 0x10
#define PRODUCT_NUMBER_LEN 0x0A
#define VERSION_LEN 0x06
#define RELEASE_LEN 0x08
#define DEVICE_INFO_LEN 0x08
#define AREA_LEN 0x0A
#define SPACE_LEN 0x06
#define PARIF_LEN 0x10
#define TITLE_LEN 0x70

#define SECURITY_CODE_LEN 0x0D00
#define AREA_CODE_LEN 0x20

#define HEADER_SIZE sizeof(saturn_header)

#define APPLICATION_INITIAL_PROGRAM_SIZE 0x71E0

typedef struct __saturn_header {
	char hardware[HARDWARE_LEN];
	char maker_id[MAKER_ID_LEN];
	char product_number[PRODUCT_NUMBER_LEN];
	char version[VERSION_LEN];
	char release[RELEASE_LEN];
	char device_info[DEVICE_INFO_LEN];
	char area[AREA_LEN];
	char space[SPACE_LEN];
	char parif[PARIF_LEN];
	char title[TITLE_LEN];
	char reserved[0x10];
	unsigned long ip_size;
	unsigned long resv;
	unsigned long stack_m;
	unsigned long stack_s;
	unsigned long first_read;
	unsigned long first_read_size;
	unsigned long resv2;
	unsigned long resv3;
} saturn_header, *saturn_header_ptr;

int read_header(int);
int validify_header();
void display_header();
int read_aip(int);

#endif
